@@ -17,6 +17,7 @@
 #include <xtl/xmeta_utils.hpp>
 #include <xtl/xsequence.hpp>
 
+#include "xaccessible.hpp"
 #include "xiterable.hpp"
 #include "xiterator.hpp"
 #include "xmath.hpp"
@@ -66,7 +67,8 @@ namespace xt
      *           provides the interface.
      */
     template <class D>
-    class xcontainer : public xcontiguous_iterable<D>
+    class xcontainer : public xcontiguous_iterable<D>,
+                       private xaccessible<D>
     {
     public:
 
@@ -76,13 +78,11 @@ namespace xt
         using storage_type = typename inner_types::storage_type;
         using allocator_type = allocator_type_t<std::decay_t<storage_type>>;
         using value_type = typename storage_type::value_type;
-        using reference = std::conditional_t<std::is_const<storage_type>::value,
-                                             typename storage_type::const_reference,
-                                             typename storage_type::reference>;
-        using const_reference = typename storage_type::const_reference;
+        using reference = typename inner_types::reference;
+        using const_reference = typename inner_types::const_reference;
         using pointer = typename storage_type::pointer;
         using const_pointer = typename storage_type::const_pointer;
-        using size_type = typename storage_type::size_type;
+        using size_type = typename inner_types::size_type;
         using difference_type = typename storage_type::difference_type;
         using simd_value_type = xsimd::simd_type<value_type>;
 
@@ -98,6 +98,8 @@ namespace xt
         using stepper = typename iterable_base::stepper;
         using const_stepper = typename iterable_base::const_stepper;
 
+        using accessible_base = xaccessible<D>;
+
         static constexpr layout_type static_layout = inner_types::layout;
         static constexpr bool contiguous_layout = static_layout != layout_type::dynamic;
         using data_alignment = xsimd::container_alignment_t<storage_type>;
@@ -127,29 +129,17 @@ namespace xt
         template <class... Args>
         const_reference operator()(Args... args) const;
 
-        template <class... Args>
-        reference at(Args... args);
-
-        template <class... Args>
-        const_reference at(Args... args) const;
-
         template <class... Args>
         reference unchecked(Args... args);
 
         template <class... Args>
         const_reference unchecked(Args... args) const;
 
-        template <class S>
-        disable_integral_t<S, reference> operator[](const S& index);
-        template <class I>
-        reference operator[](std::initializer_list<I> index);
-        reference operator[](size_type i);
-
-        template <class S>
-        disable_integral_t<S, const_reference> operator[](const S& index) const;
-        template <class I>
-        const_reference operator[](std::initializer_list<I> index) const;
-        const_reference operator[](size_type i) const;
+        using accessible_base::shape;
+        using accessible_base::at;
+        using accessible_base::operator[];
+        using accessible_base::periodic;
+        using accessible_base::in_bounds;
 
         template <class It>
         reference element(It first, It last);
@@ -169,7 +159,6 @@ namespace xt
         template <class S>
         bool has_linear_assign(const S& strides) const noexcept;
         template <class S>
-
         stepper stepper_begin(const S& shape) noexcept;
         template <class S>
         stepper stepper_end(const S& shape, layout_type l) noexcept;
@@ -234,15 +223,18 @@ namespace xt
 
     private:
 
-        template <class C>
-        friend class xstepper;
-
         template <class It>
         It data_xend_impl(It begin, layout_type l, size_type offset) const noexcept;
 
         inner_shape_type& mutable_shape();
         inner_strides_type& mutable_strides();
         inner_backstrides_type& mutable_backstrides();
+
+        template <class C>
+        friend class xstepper;
+
+        friend class xaccessible<D>;
+        friend class xconst_accessible<D>;
     };
 
     /**
@@ -286,6 +278,9 @@ namespace xt
         template <class S = shape_type>
         void reshape(S&& shape, layout_type layout = base_type::static_layout);
 
+        template <class T>
+        void reshape(std::initializer_list<T> shape, layout_type layout = base_type::static_layout);
+
         layout_type layout() const noexcept;
 
     protected:
@@ -456,40 +451,6 @@ namespace xt
         return storage()[index];
     }
 
-    /**
-     * Returns a reference to the element at the specified position in the container,
-     * after dimension and bounds checking.
-     * @param args a list of indices specifying the position in the container. Indices
-     * must be unsigned integers, the number of indices should be equal to the number of dimensions
-     * of the container.
-     * @exception std::out_of_range if the number of argument is greater than the number of dimensions
-     * or if indices are out of bounds.
-     */
-    template <class D>
-    template <class... Args>
-    inline auto xcontainer<D>::at(Args... args) -> reference
-    {
-        check_access(shape(), static_cast<size_type>(args)...);
-        return this->operator()(args...);
-    }
-
-    /**
-     * Returns a constant reference to the element at the specified position in the container,
-     * after dimension and bounds checking.
-     * @param args a list of indices specifying the position in the container. Indices
-     * must be unsigned integers, the number of indices should be equal to the number of dimensions
-     * of the container.
-     * @exception std::out_of_range if the number of argument is greater than the number of dimensions
-     * or if indices are out of bounds.
-     */
-    template <class D>
-    template <class... Args>
-    inline auto xcontainer<D>::at(Args... args) const -> const_reference
-    {
-        check_access(shape(), static_cast<size_type>(args)...);
-        return this->operator()(args...);
-    }
-
     /**
      * Returns a reference to the element at the specified position in the container.
      * @param args a list of indices specifying the position in the container. Indices
@@ -544,60 +505,6 @@ namespace xt
         return storage()[index];
     }
 
-    /**
-     * Returns a reference to the element at the specified position in the container.
-     * @param index a sequence of indices specifying the position in the container. Indices
-     * must be unsigned integers, the number of indices in the list should be equal or greater
-     * than the number of dimensions of the container.
-     */
-    template <class D>
-    template <class S>
-    inline auto xcontainer<D>::operator[](const S& index)
-        -> disable_integral_t<S, reference>
-    {
-        return element(index.cbegin(), index.cend());
-    }
-
-    template <class D>
-    template <class I>
-    inline auto xcontainer<D>::operator[](std::initializer_list<I> index) -> reference
-    {
-        return element(index.begin(), index.end());
-    }
-
-    template <class D>
-    inline auto xcontainer<D>::operator[](size_type i) -> reference
-    {
-        return operator()(i);
-    }
-
-    /**
-     * Returns a constant reference to the element at the specified position in the container.
-     * @param index a sequence of indices specifying the position in the container. Indices
-     * must be unsigned integers, the number of indices in the list should be equal or greater
-     * than the number of dimensions of the container.
-     */
-    template <class D>
-    template <class S>
-    inline auto xcontainer<D>::operator[](const S& index) const
-        -> disable_integral_t<S, const_reference>
-    {
-        return element(index.cbegin(), index.cend());
-    }
-
-    template <class D>
-    template <class I>
-    inline auto xcontainer<D>::operator[](std::initializer_list<I> index) const -> const_reference
-    {
-        return element(index.begin(), index.end());
-    }
-
-    template <class D>
-    inline auto xcontainer<D>::operator[](size_type i) const -> const_reference
-    {
-        return operator()(i);
-    }
-
     /**
      * Returns a reference to the element at the specified position in the container.
      * @param first iterator starting the sequence of indices
@@ -980,7 +887,9 @@ namespace xt
     }
 
     /**
-     * resizes the container.
+     * Resizes the container.
+     * @warning Contrary to STL containers like std::vector, resize
+     * do NOT preserve the container elements.
      * @param shape the new shape
      * @param force force reshaping, even if the shape stays the same (default: false)
      */
@@ -1004,7 +913,9 @@ namespace xt
     }
 
     /**
-     * resizes the container.
+     * Resizes the container.
+     * @warning Contrary to STL containers like std::vector, resize
+     * do NOT preserve the container elements.
      * @param shape the new shape
      * @param l the new layout_type
      */
@@ -1022,6 +933,8 @@ namespace xt
 
     /**
      * Resizes the container.
+     * @warning Contrary to STL containers like std::vector, resize
+     * do NOT preserve the container elements.
      * @param shape the new shape
      * @param strides the new strides
      */
@@ -1042,7 +955,14 @@ namespace xt
     }
 
     /**
-     * Reshapes the container and keeps old elements
+     * Reshapes the container and keeps old elements. The `shape` argument can have one of its value
+     * equal to `-1`, in this case the value is inferred from the number of elements in the container
+     * and the remaining values in the `shape`.
+     * \code{.cpp}
+     * xt::xarray<int> a = { 1, 2, 3, 4, 5, 6, 7, 8 };
+     * a.reshape({-1, 4});
+     * //a.shape() is {2, 4}
+     * \endcode
      * @param shape the new shape (has to have same number of elements as the original container)
      * @param layout the layout to compute the strides (defaults to static layout of the container,
      *               or for a container with dynamic layout to XTENSOR_DEFAULT_LAYOUT)
@@ -1054,6 +974,19 @@ namespace xt
         reshape_impl(std::forward<S>(shape), std::is_signed<std::decay_t<typename std::decay_t<S>::value_type>>(), std::forward<layout_type>(layout));
     }
 
+    template <class T>
+    struct DEBUG;
+
+    template <class D>
+    template <class T>
+    inline void xstrided_container<D>::reshape(std::initializer_list<T> shape, layout_type layout)
+    {
+        using sh_type = rebind_container_t<T, shape_type>;
+        sh_type sh = xtl::make_sequence<sh_type>(shape.size());
+        std::copy(shape.begin(), shape.end(), sh.begin());
+        reshape_impl(std::move(sh), std::is_signed<T>(), std::forward<layout_type>(layout));
+    }
+
     template <class D>
     template <class S>
     inline void xstrided_container<D>::reshape_impl(S&& shape, std::false_type /* is unsigned */, layout_type layout)
@@ -1085,7 +1018,7 @@ namespace xt
         {
             throw std::runtime_error("Negative axis size cannot be inferred. Shape mismatch.");
         }
-        std::decay_t<S> shape = _shape; 
+        std::decay_t<S> shape = _shape;
         value_type accumulator = 1;
         std::size_t neg_idx = 0;
         std::size_t i = 0;
