@@ -28,7 +28,15 @@ namespace xt
     struct xcontainer_inner_types<xdynamic_view<CT, S, L, FST>>
     {
         using xexpression_type = std::decay_t<CT>;
+        using undecay_expression = CT;
+        using reference = inner_reference_t<undecay_expression>;
+        using const_reference = typename xexpression_type::const_reference;
+        using size_type = typename xexpression_type::size_type;
+        using shape_type = std::decay_t<S>;
+        using undecay_shape = S;
+        using inner_storage_type = FST;
         using temporary_type = xarray<std::decay_t<typename xexpression_type::value_type>>;
+        static constexpr layout_type layout = L;
     };
 
     template <class CT, class S, layout_type L, class FST>
@@ -38,6 +46,11 @@ namespace xt
         using inner_strides_type = inner_shape_type;
         using inner_backstrides_type = inner_shape_type;
 
+#if defined(__GNUC__) && !defined(__clang__) && __GNUC__ == 8
+        constexpr static auto random_instantiation_var_for_gcc8_data_iface = has_data_interface<xdynamic_view<CT, S, L, FST>>::value;
+        constexpr static auto random_instantiation_var_for_gcc8_has_strides = has_strides<xdynamic_view<CT, S, L, FST>>::value;
+#endif
+
         // TODO: implement efficient stepper specific to the dynamic_view
         using const_stepper = xindexed_stepper<const xdynamic_view<CT, S, L, FST>, true>;
         using stepper = xindexed_stepper<xdynamic_view<CT, S, L, FST>, false>;
@@ -82,12 +95,12 @@ namespace xt
     class xdynamic_view : public xview_semantic<xdynamic_view<CT, S, L, FST>>,
                           public xiterable<xdynamic_view<CT, S, L, FST>>,
                           public extension::xdynamic_view_base_t<CT, S, L, FST>,
-                          private xstrided_view_base<CT, S, L, FST>
+                          private xstrided_view_base<xdynamic_view<CT, S, L, FST>>
     {
     public:
 
         using self_type = xdynamic_view<CT, S, L, FST>;
-        using base_type = xstrided_view_base<CT, S, L, FST>;
+        using base_type = xstrided_view_base<self_type>;
         using semantic_base = xview_semantic<self_type>;
         using extension_base = extension::xdynamic_view_base_t<CT, S, L, FST>;
         using expression_tag = typename extension_base::expression_tag;
@@ -163,29 +176,16 @@ namespace xt
         template <class... Args>
         const_reference operator()(Args... args) const;
 
-        template <class... Args>
-        reference at(Args... args);
-
-        template <class... Args>
-        const_reference at(Args... args) const;
-
         template <class... Args>
         reference unchecked(Args... args);
 
         template <class... Args>
         const_reference unchecked(Args... args) const;
 
-        template <class OS>
-        disable_integral_t<OS, reference> operator[](const OS& index);
-        template <class I>
-        reference operator[](std::initializer_list<I> index);
-        reference operator[](size_type i);
-
-        template <class OS>
-        disable_integral_t<OS, const_reference> operator[](const OS& index) const;
-        template <class I>
-        const_reference operator[](std::initializer_list<I> index) const;
-        const_reference operator[](size_type i) const;
+        using base_type::operator[];
+        using base_type::at;
+        using base_type::periodic;
+        using base_type::in_bounds;
 
         template <class It>
         reference element(It first, It last);
@@ -195,7 +195,12 @@ namespace xt
 
         size_type data_offset() const noexcept;
 
-        using base_type::data;
+        // Explicitly deleting data methods so has_data_interface results
+        // to false instead of having compilers complaining about not being
+        // able to call the methods from the private base
+        value_type* data() noexcept = delete;
+        const value_type* data() const noexcept = delete;
+
         using base_type::storage;
         using base_type::expression;
         using base_type::broadcast_shape;
@@ -234,16 +239,16 @@ namespace xt
 
         container_iterator data_xbegin() noexcept;
         const_container_iterator data_xbegin() const noexcept;
-        container_iterator data_xend(layout_type l) noexcept;
-        const_container_iterator data_xend(layout_type l) const noexcept;
+        container_iterator data_xend(layout_type l, size_type offset) noexcept;
+        const_container_iterator data_xend(layout_type l, size_type offset) const noexcept;
 
         template <class It>
         It data_xbegin_impl(It begin) const noexcept;
 
         template <class It>
-        It data_xend_impl(It end, layout_type l) const noexcept;
+        It data_xend_impl(It end, layout_type l, size_type offset) const noexcept;
 
-        void assign_temporary_impl(temporary_type&& tmp);      
+        void assign_temporary_impl(temporary_type&& tmp);
 
         template <class T,  class... Args>
         offset_type adjust_offset(offset_type offset, T idx, Args... args) const noexcept;
@@ -258,7 +263,9 @@ namespace xt
 
         template <class C>
         friend class xstepper;
-        friend class xview_semantic<xdynamic_view<CT, S, L, FST>>;
+        friend class xview_semantic<self_type>;
+        friend class xaccessible<self_type>;
+        friend class xconst_accessible<self_type>;
     };
 
     /**************************
@@ -429,22 +436,6 @@ namespace xt
         return base_type::storage()[static_cast<size_type>(offset)];
     }
 
-    template <class CT, class S, layout_type L, class FST>
-    template <class... Args>
-    inline auto xdynamic_view<CT, S, L, FST>::at(Args... args) -> reference
-    {
-        check_access(shape(), static_cast<size_type>(args)...);
-        return this->operator()(args...);
-    }
-
-    template <class CT, class S, layout_type L, class FST>
-    template <class... Args>
-    inline auto xdynamic_view<CT, S, L, FST>::at(Args... args) const -> const_reference
-    {
-        check_access(shape(), static_cast<size_type>(args)...);
-        return this->operator()(args...);
-    }
-
     template <class CT, class S, layout_type L, class FST>
     template <class... Args>
     inline auto xdynamic_view<CT, S, L, FST>::unchecked(Args... args) -> reference
@@ -463,46 +454,6 @@ namespace xt
         return base_type::storage()[static_cast<size_type>(offset)];
     }
 
-    template <class CT, class S, layout_type L, class FST>
-    template <class OS>
-    inline auto xdynamic_view<CT, S, L, FST>::operator[](const OS& index) -> disable_integral_t<OS, reference>
-    {
-        return element(index.cbegin(), index.cend());
-    }
-
-    template <class CT, class S, layout_type L, class FST>
-    template <class I>
-    inline auto xdynamic_view<CT, S, L, FST>::operator[](std::initializer_list<I> index) -> reference
-    {
-        return element(index.begin(), index.end());
-    }
-
-    template <class CT, class S, layout_type L, class FST>
-    inline auto xdynamic_view<CT, S, L, FST>::operator[](size_type i) -> reference
-    {
-        return operator()(i);
-    }
-
-    template <class CT, class S, layout_type L, class FST>
-    template <class OS>
-    inline auto xdynamic_view<CT, S, L, FST>::operator[](const OS& index) const -> disable_integral_t<OS, const_reference>
-    {
-        return element(index.cbegin(), index.cend());
-    }
-
-    template <class CT, class S, layout_type L, class FST>
-    template <class I>
-    inline auto xdynamic_view<CT, S, L, FST>::operator[](std::initializer_list<I> index) const -> const_reference
-    {
-        return element(index.begin(), index.end());
-    }
-
-    template <class CT, class S, layout_type L, class FST>
-    inline auto xdynamic_view<CT, S, L, FST>::operator[](size_type i) const -> const_reference
-    {
-        return operator()(i);
-    }
-
     template <class CT, class S, layout_type L, class FST>
     template <class It>
     inline auto xdynamic_view<CT, S, L, FST>::element(It first, It last) -> reference
@@ -578,7 +529,7 @@ namespace xt
         inner_strides_type str(base_type::strides());
         slice_vector_type svt(m_slices);
         inner_strides_type adj_str(m_adj_strides);
-        return rebind_t<E>(std::forward<E>(e), std::move(sh), std::move(str), 
+        return rebind_t<E>(std::forward<E>(e), std::move(sh), std::move(str),
                            base_type::data_offset(), this->layout(), std::move(svt), std::move(adj_str));
     }
 
@@ -595,15 +546,15 @@ namespace xt
     }
 
     template <class CT, class S, layout_type L, class FST>
-    inline auto xdynamic_view<CT, S, L, FST>::data_xend(layout_type l) noexcept -> container_iterator
+    inline auto xdynamic_view<CT, S, L, FST>::data_xend(layout_type l, size_type offset) noexcept -> container_iterator
     {
-        return data_xend_impl(this->storage().begin(), l);
+        return data_xend_impl(this->storage().begin(), l, offset);
     }
 
     template <class CT, class S, layout_type L, class FST>
-    inline auto xdynamic_view<CT, S, L, FST>::data_xend(layout_type l) const noexcept -> const_container_iterator
+    inline auto xdynamic_view<CT, S, L, FST>::data_xend(layout_type l, size_type offset) const noexcept -> const_container_iterator
     {
-        return data_xend_impl(this->storage().cbegin(), l);
+        return data_xend_impl(this->storage().cbegin(), l, offset);
     }
 
     template <class CT, class S, layout_type L, class FST>
@@ -617,10 +568,9 @@ namespace xt
 
     template <class CT, class S, layout_type L, class FST>
     template <class It>
-    inline It xdynamic_view<CT, S, L, FST>::data_xend_impl(It begin, layout_type l) const noexcept
+    inline It xdynamic_view<CT, S, L, FST>::data_xend_impl(It begin, layout_type l, size_type offset) const noexcept
     {
-        std::ptrdiff_t end_offset = static_cast<std::ptrdiff_t>(std::accumulate(this->backstrides().begin(), this->backstrides().end(), std::size_t(0)));
-        return strided_data_end(*this, begin + std::ptrdiff_t(data_offset()) + end_offset + 1, l);
+        return strided_data_end(*this, begin + std::ptrdiff_t(data_offset()), l, offset);
     }
 
     template <class CT, class S, layout_type L, class FST>
